class MERResourceGroupUtilizationPlus(PWReportPage):
	def init(self):
		super(MERResourceGroupUtilizationPlus, self).init()
		self.project				= None
		self.withObjective 			= True
		self.withResources			= True
		self.withMasterResources	= True
		self.intervalUnit			= 0
		self.isCumulated            = False
		self.graphColor             = None
		self.contentMode			= 0
		self.contentType            = 0
		self.iterIntervalStartDate	= None
		self.withTimes 		 		= True
		self.withTotals             = True
		self.withWorkCost   		= False
		self.withAssignments        = False
		self.iterItem				= None
		self.iterIndex				= 0
		self.iterTitle			    = None
		self.iterTitleIndex	        = 0
		self.iterTask			    = None
		self.iterTaskIndex	        = 0
		self.milestonesSortDescriptors			= [NSSortDescriptor.alloc().initWithKey_ascending_("expectedEndDate", True)]
		self.upcomingActivitiesSortDescriptors	= [NSSortDescriptor.alloc().initWithKey_ascending_("expectedStartDate", True)]
		self.overdueActivitiesSortDescriptors	= [NSSortDescriptor.alloc().initWithKey_ascending_("expectedStartDate", True)]
		self.deadlineActivitiesSortDescriptors	= [NSSortDescriptor.alloc().initWithKey_ascending_("expectedStartDate", True)]
		self.infoElementsSortDescriptors		= [NSSortDescriptor.alloc().initWithKey_ascending_("informationDate", True)]
		self.issuesSortDescriptors				= [NSSortDescriptor.alloc().initWithKey_ascending_("fixUntilDate", True)]
		self.risksSortDescriptors				= [NSSortDescriptor.alloc().initWithKey_ascending_("recordingDate", True)]
		self.groupsSortDescriptors			= [NSSortDescriptor.alloc().initWithKey_ascending_("title", True)]
		self.rolesSortDescriptors			= [NSSortDescriptor.alloc().initWithKey_ascending_("title", True)]
		self.masterResourcesSortDescriptors		= [NSSortDescriptor.alloc().initWithKey_ascending_("title", True)]
		self.filesSortDescriptors				= [NSSortDescriptor.alloc().initWithKey_ascending_("title", True)]
		return self

	def internalParameterKeys(self):
		return ["groupsSortDescriptors", "rolesSortDescriptors", "resourcesSortDescriptors"]

	def withGroups(self):
		return (self.contentMode == 0)
			
	def withRoles(self):
		return (self.contentMode == 1)
	
	def withUnits(self):
		return (self.contentType == 0)

	def resources(self):
		predicate = NSPredicate.predicateWithFormat_argumentArray_("isPrivate = FALSE", None)
		resources = self.project.visibleResources().allObjects().filteredArrayUsingPredicate_(predicate)
		resources = resources.sortedArrayUsingDescriptors_(self.resourcesSortDescriptors)
		return resources
		
	def roles(self):
		roles = self.project.roles()
		roles = roles.sortedArrayUsingDescriptors_(self.rolesSortDescriptors)
		return roles
		
	def groups(self):
		groups = self.project.resourceGroups()
		groups = groups.sortedArrayUsingDescriptors_(self.groupsSortDescriptors)
		return groups

	
	def showObjective(self):
		return self.withObjective and self.project.itemDescription()



	def rowClass(self):
		if self.iterIndex%2:
			return "even"

	def listItemClass(self):
		if self.iterIndex == 0:
			return "first"
	
	def dateFormat(self):
		if self.withTimes:
			return "medium/short"
		else:
			return "medium"

	def xTicks(self):
		xTicks = "\"xTicks\": [ "
		dates = self.intervalStartDates()		
		for i in range(len(dates)):
			cat = MEDuration.preciseDescriptionForUnit_atDate_calendar_(self.intervalUnit, dates[i], self.project.sharedNSCalendar())
			if i<len(dates)-1:
				separator = ","
			else:
				separator = ""
			titel = cat.replace("\"", "\\\"")
			xTicks += "{v:%d, label:\"%s\"}%s\n" % (i, titel, separator)
		xTicks = xTicks + "]"
		return xTicks
	
	def dataSetStringForKey(self, key):
		values = []
		result = 0
		if self.withGroups():
			contents = self.groups()
		else:
			contents = self.roles()
		dataSetString = "\"%s\",[" % (key)
		dates = self.intervalStartDates()
		for i in range(len(dates)):
			result = 0
			start = dates[i]
			end = MEDuration.endOfUnit_withStartDate_calendar_(self.intervalUnit, start, self.project.sharedNSCalendar())
			
			if contents:
				for aItem in contents:					
					res = aItem.valueForKey_("resources")
					if res:
						for re in res:
							assignements = re.valueForKey_("assignments")
							if assignements:
								for a in assignements:
									if a.expectedWorkingUnitsBetweenStartDate_endDate_(start, end):
										if self.contentType == 0:
											result += 1		
											break
										else:
											result += a.expectedWorkingUnitsBetweenStartDate_endDate_(start, end)
			
				if result != None: 
					value = "%.1f" % (result/self.project.manDayHours())
				else:
					value = 0
				if i<len(dates)-1:
					separator = ","
				else:
					separator = ""
				dataSetString += "[%d, %s]%s" % (i, value, separator)
		dataSetString += "]"
		return dataSetString

	def statusStatsDataSetString(self):
		return self.dataSetStringForKey("work")
		
		
	def graphColorString(self):
		return self.graphColor.webColor()

	def graphUnit(self):
		if self.contentType == 0:
			return " units"
		else:
			return " days"

			
	def intervalUnitIsTooSmall(self):
		if self.project.expectedEndDate() and self.project.expectedStartDate():
			projectSeconds = self.project.expectedEndDate().timeIntervalSinceDate_(self.project.expectedStartDate())
			intervalSeconds = MEDuration.durationWithAmount_unit_isFloating_relativeError_(1, self.intervalUnit, 1, 0).roughSeconds()
			numberOfIntervals = projectSeconds / intervalSeconds
			return numberOfIntervals > 300
		else:
			return False
	
	def shouldIncludeStartDate(self, startDate):
		endDate =  MEDuration.endOfUnit_withStartDate_calendar_(self.intervalUnit, startDate, self.project.sharedNSCalendar())
		return True
	
	def intervalStartDates(self):
		start = MEDuration.startOfUnit_containingDate_calendar_(self.intervalUnit, self.project.expectedStartDate(), self.project.sharedNSCalendar())
		end   = self.project.expectedEndDate()
		
		intervalStart = start
		dates = []
		
		while intervalStart<end:
			if self.shouldIncludeStartDate(intervalStart):
				dates.append(intervalStart)
			intervalStart = MEDuration.endOfUnit_withStartDate_calendar_(self.intervalUnit, intervalStart, self.project.sharedNSCalendar())
			
		return dates
		
	def setIterIntervalStartDate_(self, date):
		self.iterIntervalStartDate = date
		self.iterIntervalEndDate = MEDuration.endOfUnit_withStartDate_calendar_(self.intervalUnit, date, self.project.sharedNSCalendar())
		
		
	def iterIntervalDescription(self):
		return MEDuration.preciseDescriptionForUnit_atDate_calendar_(self.intervalUnit, self.iterIntervalStartDate, self.project.sharedNSCalendar())

	def iterIntervalCalculcationStartDate(self):
		if self.isCumulated:
			return self.project.expectedStartDate()
		else:
			return self.iterIntervalStartDate

	def iterIntervalExpectedWorkCost(self):
		return self.project.expectedWorkCostBetweenDate_andDate_(self.iterIntervalCalculcationStartDate(), self.iterIntervalEndDate)
		
	def iterIntervalExpectedWorkTotalByGroup(self):
		result = 0
		groups = self.groups()
		if groups:
			for group in groups:
				res = group.valueForKey_("resources")
				if res:
					for re in res:
						handling = None
						assignements = re.valueForKey_("assignments")
						if assignements:
							for a in assignements:
								if a.expectedWorkingUnitsBetweenStartDate_endDate_(self.iterIntervalCalculcationStartDate(), self.iterIntervalEndDate):
									if self.contentType == 0:
										result += 1		
										break
									else:
										result += a.expectedWorkingUnitsBetweenStartDate_endDate_(self.iterIntervalCalculcationStartDate(), self.iterIntervalEndDate)
			if self.contentType == 0:
				return result
			else:
				return 	"%1.2f" % (result/self.project.manDayHours())
			

	def iterIntervalExpectedWorkTotalByRole(self):
		result = 0
		roles = self.roles()
		if roles:
			for role in roles:
				res = role.valueForKey_("resources")
				if res:
					for re in res:
						assignements = re.valueForKey_("assignments")
						if assignements:
							for a in assignements:
								if a.expectedWorkingUnitsBetweenStartDate_endDate_(self.iterIntervalCalculcationStartDate(), self.iterIntervalEndDate):
									if self.contentType == 0:
										result += 1		
										break
									else:
										result += a.expectedWorkingUnitsBetweenStartDate_endDate_(self.iterIntervalCalculcationStartDate(), self.iterIntervalEndDate)
			if self.contentType == 0:
				return result
			else:
				return 	"%1.2f" % (result/self.project.manDayHours())
			
			

	def iterIntervalExpectedWorkPerGroup(self):
		result = 0
		if self.iterItem != None:
			res = self.iterItem.valueForKey_("resources")
			if res:
				for re in res:
					assignements = re.valueForKey_("assignments")
					if assignements:
						for a in assignements:
							if a.expectedWorkingUnitsBetweenStartDate_endDate_(self.iterIntervalCalculcationStartDate(), self.iterIntervalEndDate):
								if self.contentType == 0:
									result += 1		
									break
								else:
									result += a.expectedWorkingUnitsBetweenStartDate_endDate_(self.iterIntervalCalculcationStartDate(), self.iterIntervalEndDate)
								
			if self.contentType == 0:
				return result
			else:
				return 	"%1.2f" % (result/self.project.manDayHours())
			

